# -------------------------------------------------------------------------------------------------
# test_manager1_mp_xi2.py: Multiprocessing and Manager Objects                              
#
# Theory:      Server processes and Manager objects
#
# Description: A manager object returned by Manager()
#               controls a server process 
#                        which holds Python objects and allows other processes to manipulate them using proxies.
#               supports types list, dict, Namespace, Lock, RLock, Semaphore, BoundedSemaphore, Condition, 
#                        Event, Queue, Value and Array
#
#               acquires a lock before processing and releases that lock after completing
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Imports from standard modules                              
# -------------------------------------------------------------------------------------------------
import os
import sys


# -------------------------------------------------------------------------------------------------
# Imports from multiprocessing package                              
# -------------------------------------------------------------------------------------------------
# from multiprocessing import Process, Queue, Pipe, Lock, Manager
from multiprocessing.process import Process
from multiprocessing.queues  import Queue
from multiprocessing.connection import Pipe
from multiprocessing.synchronize import Lock
from multiprocessing import Manager #<--- this is because Manager function is defined in __init__.py module!

# -------------------------------------------------------------------------------------------------
# yourDragonXi imports                               
# -------------------------------------------------------------------------------------------------
from process2xi import Process2Xi
from queue2xi  import Queue2Xi
from pipe2xi import Pipe2Xi
from lock2xi import Lock2Xi
from manager2xi import Manager2Xi #<--- this the new module created for this function


# -------------------------------------------------------------------------------------------------
# sense function for multiprocessing
# -------------------------------------------------------------------------------------------------
#
def sense(target_param, queue_param, pipe_param,lock_param,dict_param,list_param):
    target_xi2 = target_param                    # target to be sensed
    queue_xi2 = queue_param                      # queue
    pipe_publisher_channel_xi2 = pipe_param      # pipe
    lock_xi2 = lock_param                        # lock for resources
    dict_xi2 = dict_param                        # dictionary created by the manager
    list_xi2 = list_param                        # list created by the manager
    
    
    print("SENSE: ", target_xi2)
    # ---------------------------------------------------------------------------------------------
    # put a message to the queue
    # 
    queue_xi2.put(['DragonXi2_00', target_xi2, 'sensing!']) # puts to its queue
    # ---------------------------------------------------------------------------------------------
    # reserve the pipe using the lock, send messages and release the pipe
    # 
    lock_xi2.acquire()

    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_01', 'BIO ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_02', 'CO2 ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_03', 'Coal ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_04', 'Diesel ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_05', 'Electricity ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_06', 'Fusion ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_07', 'Gas ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_08', 'Geo ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_09', 'Hydrogen ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_10', 'Lithium ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_11', 'Nuclear Power ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_12', 'Ocean ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_13', 'Oil ',target_xi2]) 
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_14', 'Solar ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_15', 'Wind ',target_xi2])
    
    dict_xi2[1] = "BIO"
    dict_xi2[2] = "CO2"
    dict_xi2[3] = "Coal"
    dict_xi2[4] = "Diesel"
    dict_xi2[5] = "Electricity"
    dict_xi2[6] = "Fusion"
    dict_xi2[7] = "Gas"
    dict_xi2[8] = "Geo"
    dict_xi2[9] = "Hydrogen"
    dict_xi2[10] = "Lithium"
    dict_xi2[11] = "Nuclear Power"
    dict_xi2[12] = "Ocean"
    dict_xi2[13] = "Oil"
    dict_xi2[14] = "Solar"
    dict_xi2[15] = "Wind"
    

    lock_xi2.release()       
# -------------------------------------------------------------------------------------------------
# reason funcion for multiprocessing
# -------------------------------------------------------------------------------------------------
# 
def reason(target_param, queue_param, pipe_param,lock_param,dict_param,list_param):
    target_xi4 = target_param
    queue_xi4 = queue_param
    pipe_publisher_channel_xi4 = pipe_param
    lock_xi4 = lock_param
    dict_xi4 = dict_param
    list_xi8 = list_param
    
    print("REASON: ", target_xi4)
    # ---------------------------------------------------------------------------------------------
    # put a message to the queue
    # 
    queue_xi4.put(['DragonXi4_00', target_xi4, 'portfolio must be sustainable!']) # puts to its queue 
    # ---------------------------------------------------------------------------------------------
    # reserve the pipe using the lock, send messages and release the pipe
    # 
    lock_xi4.acquire()
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_01', 'BIO ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_02', 'Geo ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_03', 'Hydrogen ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_04', 'Nuclear Power ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_05', 'Ocean ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_06', 'Solar ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_07', 'Wind ',target_xi4])    

    dict_xi4[1] = "BIO"
    dict_xi4[2] = "Geo"
    dict_xi4[3] = "Hydrogen"
    dict_xi4[4] = "Nuclear Power"
    dict_xi4[5] = "Ocean"
    dict_xi4[6] = "Solar"
    dict_xi4[7] = "Wind"
    
    lock_xi4.release()
        
# -------------------------------------------------------------------------------------------------
# ace function for multiprocessing
# -------------------------------------------------------------------------------------------------
#
def ace(target_param, queue_param, pipe_param,lock_param,dict_param,list_param):
    # ---------------------------------------------------------------------------------------------
    # 
    # 
    target_xi8 = target_param
    queue_xi8 = queue_param
    pipe_publisher_channel_xi8 = pipe_param
    lock_xi8 = lock_param
    dict_xi8 = dict_param
    list_xi8 = list_param
    
    print("ACE: ", target_xi8)
    # ---------------------------------------------------------------------------------------------
    # put a message to the queue
    # 
    queue_xi8.put(['DragonXi8_00', target_xi8, 'to be aced!']) # puts to its queue       
    
    # ---------------------------------------------------------------------------------------------
    # reserve the pipe using the lock, send messages and release the pipe
    # 
    lock_xi8.acquire()
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_01', 'BIO ',target_xi8])
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_02', 'Hydrogen ',target_xi8])
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_03', 'Nuclear Power ',target_xi8])
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_04', 'Solar ',target_xi8])
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_05', 'Wind ',target_xi8])    

    dict_xi8[1] = "BIO"
    dict_xi8[2] = "Hydrogen"
    dict_xi8[3] = "Nuclear Power"
    dict_xi8[4] = "Solar"
    dict_xi8[5] = "Wind"
    
    lock_xi8.release()

# -------------------------------------------------------------------------------------------------
# phase1 function uses the original multiprocessing modules
# -------------------------------------------------------------------------------------------------
#
def phase1():
    # =============================================================================================
    # PHASE I: use the original multiprocessing modules
    # =============================================================================================
    # ---------------------------------------------------------------------------------------------
    # create multiprocessing queues for sensing, reasoning and acing
    # 
    queue2xi = Queue()
    queue4xi = Queue()
    queue8xi = Queue()
    # ---------------------------------------------------------------------------------------------
    # create multiprocessing pipes with consumer and publisher ends
         
    pipe2xi_consumer_channel, pipe2xi_publisher_channel = Pipe()
    pipe4xi_consumer_channel, pipe4xi_publisher_channel = Pipe()
    pipe8xi_consumer_channel, pipe8xi_publisher_channel = Pipe()
    # ---------------------------------------------------------------------------------------------
    # create three locks
    #   
    lock_xi2 = Lock()
    lock_xi4 = Lock()
    lock_xi8 = Lock()    
    # ---------------------------------------------------------------------------------------------
    # create three managers
           
    manager2xi = Manager()
    manager4xi = Manager()
    manager8xi = Manager()
    # ---------------------------------------------------------------------------------------------
    # create three dictionaries using each manager
          
    dict2xi = manager2xi.dict()
    dict4xi = manager4xi.dict()
    dict8xi = manager8xi.dict()
    
    # ---------------------------------------------------------------------------------------------
    # create three lists using each manager
          
    list2xi = manager2xi.list(range(15))
    list4xi = manager4xi.list(range(7))
    list8xi = manager8xi.list(range(5))
    
    # ---------------------------------------------------------------------------------------------
    # tests with different values to test 8 giga RAM
    #  
    
    #list2xi = manager2xi.list(range(4294967206)) #<--- too much
    #list2xi = manager2xi.list(range(2147383603)) #<--- memory error
    #list2xi = manager2xi.list(range(32767)) #<--- 
    #list2xi = manager2xi.list(range(65534)) #<---
    #list2xi = manager2xi.list(range(131068)) #<---
    #list2xi = manager2xi.list(range(393204)) #<--- 1.72 gigabytes in Task Manager
    #list4xi = manager4xi.list(range(393204))
    #list8xi = manager8xi.list(range(393204))
    #list2xi = manager2xi.list(range(7865408)) #<--- 3 gigabytes in Task Manager
    
    #list2xi = manager2xi.list(range(1572816)) #<---  1.8 gigabytes in Task Manager !
    #list4xi = manager4xi.list(range(1572816))
    #list8xi = manager8xi.list(range(1572816))
    
    #list2xi = manager2xi.list(range(3145632)) #<---  gigabytes in Task Manager
    #list4xi = manager4xi.list(range(3145632))
    #list8xi = manager8xi.list(range(3145632))
    
    #list2xi = manager2xi.list(range(1000000000)) #<---  7.86 gigabytes in Task Manager ==> Vista tilted !
    #list4xi = manager4xi.list(range(7))
    #list8xi = manager8xi.list(range(5))
    
    
    # ---------------------------------------------------------------------------------------------
    # SENSE IT!
    #          create a process and use its services such the content it sends via its queue and pipe    
    #
    print("\n~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~")
    process2xi = Process(target=sense, args=('Energy ',queue2xi, pipe2xi_publisher_channel,lock_xi2,dict2xi,list2xi))
    process2xi.start()
    print (queue2xi.get())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
     
    print("\n Dictionary created by manager2xi and updated by process2xi:")
    print (dict2xi)
    
    print("\n List for manager2xi and process2xi:")
    print (list2xi)
     
    process2xi.join()
    
    # ---------------------------------------------------------------------------------------------
    # REASON IT!
    #         create a process and use its services such the content it sends via its queue and pipe    
    # 
    print("\n~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~")
    process4xi = Process(target=reason, args=('Energy ',queue4xi, pipe4xi_publisher_channel,lock_xi4,dict4xi,list8xi))
    process4xi.start()
    print (queue4xi.get())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    
    print("\n Dictionary created by manager4xi and updated by process4xi:")
    print (dict4xi)
    
    print("\n List for manager4xi and process4xi:")
    print (list4xi)
    
    process4xi.join()
    
    # ---------------------------------------------------------------------------------------------
    # ACE IT!
    #         create a process and use its services such the content it sends via its queue and pipe    
    # 
    print("\n~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~")
    process8xi = Process(target=ace, args=('Energy ',queue8xi, pipe8xi_publisher_channel,lock_xi8,dict8xi,list8xi))
    process8xi.start()
    print (queue8xi.get())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
     
    print("\n Dictionary created by manager8xi and updated by process8xi:")
    print (dict8xi)
     
    print("\n List for manager8xi and process8xi:")
    print (list8xi)
     
    process8xi.join()
    # ---------------------------------------------------------------------------------------------
    # Notes:
    #            above assumed the main to know amount of messages
    #            no locks were used
    print("\n===============================================================")     
    
# -------------------------------------------------------------------------------------------------
# phase2 function uses yourDragonXi modules
# -------------------------------------------------------------------------------------------------
#
def phase2():
    # =============================================================================================
    # PHASE II: using yourDragonXi modules
    # =============================================================================================
    # ---------------------------------------------------------------------------------------------
    # create multiprocessing queues for sensing, reasoning and acing
    # 
    queue2xi = Queue2Xi()
    queue4xi = Queue2Xi()
    queue8xi = Queue2Xi()
    # ---------------------------------------------------------------------------------------------
    # create multiprocessing pipes with consumer and publisher ends
         
    pipe2xi_consumer_channel, pipe2xi_publisher_channel = Pipe2Xi()
    pipe4xi_consumer_channel, pipe4xi_publisher_channel = Pipe2Xi()
    pipe8xi_consumer_channel, pipe8xi_publisher_channel = Pipe2Xi()
    # ---------------------------------------------------------------------------------------------
    # create three locks
    #   
    lock_xi2 = Lock2Xi()
    lock_xi4 = Lock2Xi()
    lock_xi8 = Lock2Xi()    
    # ---------------------------------------------------------------------------------------------
    # create three managers
    # 
    manager2xi = Manager2Xi()
    manager4xi = Manager2Xi()
    manager8xi = Manager2Xi()
    # ---------------------------------------------------------------------------------------------
    # create three dictionaries using each manager
          
    dict2xi = manager2xi.dict()
    dict4xi = manager4xi.dict()
    dict8xi = manager8xi.dict()
    
    # ---------------------------------------------------------------------------------------------
    # create three lists using each manager
          
    list2xi = manager2xi.list(range(15))
    list4xi = manager4xi.list(range(7))
    list8xi = manager8xi.list(range(5))
    
   
    
    # ---------------------------------------------------------------------------------------------
    # SENSE IT!
    #           create a process and use its services such the content it sends via its queue and pipe    
    #
    print("\n with yourDragonXi SENSE IT!")
    process2xi = Process2Xi(target=sense, args=('Energy ',queue2xi, pipe2xi_publisher_channel,lock_xi2,dict2xi,list2xi))
    process2xi.start()
    print (queue2xi.get())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    
    print("\n Dictionary created by manager2xi and updated by process2xi:")
    print (dict2xi)
    
    print("\n List for manager2xi and process2xi:")
    print (list2xi)
    
    process2xi.join()
    
    # ---------------------------------------------------------------------------------------------
    # REASON IT!
    #        create a process and use its services such the content it sends via its queue and pipe    
    # 
    print("\n with yourDragonXi REASON IT!")
    process4xi = Process2Xi(target=reason, args=('Energy ',queue4xi, pipe4xi_publisher_channel,lock_xi4,dict4xi,list8xi))
    process4xi.start()
    print (queue4xi.get())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    
    print("\n Dictionary created by manager4xi and updated by process4xi:")
    print (dict4xi)
    
    print("\n List for manager4xi and process4xi:")
    print (list4xi)
    
    process4xi.join()
    
    # ---------------------------------------------------------------------------------------------
    # ACE IT!
    #        create a process and use its services such the content it sends via its queue and pipe    
    # 
    print("\n with yourDragonXi ACE IT!")
    process8xi = Process2Xi(target=ace, args=('Energy ',queue8xi, pipe8xi_publisher_channel,lock_xi8,dict8xi,list8xi))
    process8xi.start()
    print (queue8xi.get())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    
    print("\n Dictionary created by manager8xi and updated by process8xi:")
    print (dict8xi)
    
    print("\n List for manager8xi and process8xi:")
    print (list8xi)
    
    process8xi.join()
    # ---------------------------------------------------------------------------------------------
    # Notes:
    #       above assumed the main to know amount of messages
    #       no locks were used
    print("\n===============================================================") 
           
        
# -------------------------------------------------------------------------------------------------
# main function                              
# -------------------------------------------------------------------------------------------------
def main():

    phase1()
    
    phase2()    


# -------------------------------------------------------------------------------------------------
# main                              
# -------------------------------------------------------------------------------------------------
if __name__ == '__main__':
    main()



# -------------------------------------------------------------------------------------------------
# end of test_manager1_mp_xi2.py                              
# ------------------------------------------------------------------------------------------------- 

# -------------------------------------------------------------------------------------------------
# Changes 14.11.2008 13:25                              
# ------------------------------------------------------------------------------------------------- 
# process2xi.py: all methods with @property had to be changed as follows
#     @property
#     def name(self):
#         '''
#         Process2Xi name method: returns process name
#         '''
#         #return Process.name(self) # property object is not callable!
#         #therefore we must copy the code from the original module
#         return self._name
# -------------------------------------------------------------------------------------------------
# Test 14.11.2008 13:25 Output                              
# ------------------------------------------------------------------------------------------------- 
# 
# SENSE:  Energy 
# REASON:  Energy 
# ACE:  Energy 
# 
# ~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~
# ['DragonXi2_00', 'Energy ', 'sensing!']
# ['DragonXi2_Sensing_01', 'BIO ', 'Energy ']
# ['DragonXi2_Sensing_02', 'CO2 ', 'Energy ']
# ['DragonXi2_Sensing_03', 'Coal ', 'Energy ']
# ['DragonXi2_Sensing_04', 'Diesel ', 'Energy ']
# ['DragonXi2_Sensing_05', 'Electricity ', 'Energy ']
# ['DragonXi2_Sensing_06', 'Fusion ', 'Energy ']
# ['DragonXi2_Sensing_07', 'Gas ', 'Energy ']
# ['DragonXi2_Sensing_08', 'Geo ', 'Energy ']
# ['DragonXi2_Sensing_09', 'Hydrogen ', 'Energy ']
# ['DragonXi2_Sensing_10', 'Lithium ', 'Energy ']
# ['DragonXi2_Sensing_11', 'Nuclear Power ', 'Energy ']
# ['DragonXi2_Sensing_12', 'Ocean ', 'Energy ']
# ['DragonXi2_Sensing_13', 'Oil ', 'Energy ']
# ['DragonXi2_Sensing_14', 'Solar ', 'Energy ']
# ['DragonXi2_Sensing_15', 'Wind ', 'Energy ']
# 
#  Dictionary created by manager2xi and updated by process2xi:
# {1: 'BIO', 2: 'CO2', 3: 'Coal', 4: 'Diesel', 5: 'Electricity'}
# 
#  List for manager2xi and process2xi:
# [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]
# 
# ~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~
# ['DragonXi4_00', 'Energy ', 'portfolio must be sustainable!']
# ['DragonXi4_Reasoning_01', 'BIO ', 'Energy ']
# ['DragonXi4_Reasoning_02', 'Geo ', 'Energy ']
# ['DragonXi4_Reasoning_03', 'Hydrogen ', 'Energy ']
# ['DragonXi4_Reasoning_04', 'Nuclear Power ', 'Energy ']
# ['DragonXi4_Reasoning_05', 'Ocean ', 'Energy ']
# ['DragonXi4_Reasoning_06', 'Solar ', 'Energy ']
# ['DragonXi4_Reasoning_07', 'Wind ', 'Energy ']
# 
#  Dictionary created by manager4xi and updated by process4xi:
# {1: 'BIO', 2: 'Geo', 3: 'Hydrogen', 4: 'Nuclear Power', 5: 'Ocean'}
# 
#  List for manager4xi and process4xi:
# [0, 1, 2, 3, 4, 5, 6]
# 
# ~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~
# ['DragonXi8_00', 'Energy ', 'to be aced!']
# ['DragonXi8_Ace_01', 'BIO ', 'Energy ']
# ['DragonXi8_Ace_02', 'Hydrogen ', 'Energy ']
# ['DragonXi8_Ace_03', 'Nuclear Power ', 'Energy ']
# ['DragonXi8_Ace_04', 'Solar ', 'Energy ']
# ['DragonXi8_Ace_05', 'Wind ', 'Energy ']
# 
#  Dictionary created by manager8xi and updated by process8xi:
# {1: 'BIO', 2: 'Hydrogen', 3: 'Nuclear Power'}
# 
#  List for manager8xi and process8xi:
# [0, 1, 2, 3, 4]
# 
# ===============================================================
# 
#  with yourDragonXi SENSE IT!
# ['DragonXi2_00', 'Energy ', 'sensing!']
# ['DragonXi2_Sensing_01', 'BIO ', 'Energy ']
# ['DragonXi2_Sensing_02', 'CO2 ', 'Energy ']
# ['DragonXi2_Sensing_03', 'Coal ', 'Energy ']
# ['DragonXi2_Sensing_04', 'Diesel ', 'Energy ']
# ['DragonXi2_Sensing_05', 'Electricity ', 'Energy ']
# ['DragonXi2_Sensing_06', 'Fusion ', 'Energy ']
# ['DragonXi2_Sensing_07', 'Gas ', 'Energy ']
# ['DragonXi2_Sensing_08', 'Geo ', 'Energy ']
# ['DragonXi2_Sensing_09', 'Hydrogen ', 'Energy ']
# ['DragonXi2_Sensing_10', 'Lithium ', 'Energy ']
# ['DragonXi2_Sensing_11', 'Nuclear Power ', 'Energy ']
# ['DragonXi2_Sensing_12', 'Ocean ', 'Energy ']
# ['DragonXi2_Sensing_13', 'Oil ', 'Energy ']
# ['DragonXi2_Sensing_14', 'Solar ', 'Energy ']
# ['DragonXi2_Sensing_15', 'Wind ', 'Energy ']
# 
#  Dictionary created by manager2xi and updated by process2xi:
# {}
# 
#  List for manager2xi and process2xi:
# [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]
# 
#  with yourDragonXi REASON IT!
# ['DragonXi4_00', 'Energy ', 'portfolio must be sustainable!']
# ['DragonXi4_Reasoning_01', 'BIO ', 'Energy ']
# ['DragonXi4_Reasoning_02', 'Geo ', 'Energy ']
# ['DragonXi4_Reasoning_03', 'Hydrogen ', 'Energy ']
# ['DragonXi4_Reasoning_04', 'Nuclear Power ', 'Energy ']
# ['DragonXi4_Reasoning_05', 'Ocean ', 'Energy ']
# ['DragonXi4_Reasoning_06', 'Solar ', 'Energy ']
# ['DragonXi4_Reasoning_07', 'Wind ', 'Energy ']
# 
#  Dictionary created by manager4xi and updated by process4xi:
# {}
# 
#  List for manager4xi and process4xi:
# [0, 1, 2, 3, 4, 5, 6]
# 
#  with yourDragonXi ACE IT!
# ['DragonXi8_00', 'Energy ', 'to be aced!']
# ['DragonXi8_Ace_01', 'BIO ', 'Energy ']
# ['DragonXi8_Ace_02', 'Hydrogen ', 'Energy ']
# ['DragonXi8_Ace_03', 'Nuclear Power ', 'Energy ']
# ['DragonXi8_Ace_04', 'Solar ', 'Energy ']
# ['DragonXi8_Ace_05', 'Wind ', 'Energy ']
# 
#  Dictionary created by manager8xi and updated by process8xi:
# {}
# 
#  List for manager8xi and process8xi:
# [0, 1, 2, 3, 4]
# 
# ===============================================================